package mobicomp.hearts.communication;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/**
 * @author Andreas Wetzel <andi@spamato.net>
 * Last update 8.1.2005 Nicolas
 * 
 * This class is used by the HeartsGame and the lobby for all
 * communication.
 */

public class HeartsMessage implements Serializable
{
    public static final int LOOK_FOR_GAMES = 0x01;
    public static final int HAVE_GAME = 0x02;
    public static final int JOIN_GAME_REQ = 0x03;
    public static final int JOIN_GAME_ACK = 0x04;
    public static final int LEAVE_GAME = 0x05;
    public static final int ABORT_GAME = 0x06;
    public static final int START_GAME = 0x07;

    private int type;
    Object payload;

    public HeartsMessage( int type, Object payload) {
        this.type = type;
        this.payload = payload;
    }

    public int getType() {
        return type;
    }

    public Object getPayload() {
        return payload;
    }

    /**
     * This method is used to serialize a Hearts message. The resulting
     * byte array can be used as data of a MHMessage
     * 
     * @return The HeartsMessage serialized as a byte array
     */
    public byte[] toByteArray() {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        DataOutputStream out = new DataOutputStream( bos);
        ObjectOutputStream objout;
        try {
            objout = new ObjectOutputStream( bos);
            out.writeShort( type);
            objout.writeObject( payload);
        }
        catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        return bos.toByteArray();
    }

    /**
     * This method is used to deserialize a HeartsMessage received
     * as data of a MHMessage
     * 
     * @param buffer The serialized form of the HeartsMessage
     * @return The deserialized HeartsMessage
     */
    public static HeartsMessage parseBytes( byte[] buffer) {
        ByteArrayInputStream bis = new ByteArrayInputStream( buffer);
        DataInputStream in = new DataInputStream( bis);
        ObjectInputStream objin;
        try {
            objin = new ObjectInputStream( bis);
            int type = in.readShort();
            Object payload = objin.readObject();

            return new HeartsMessage( type, payload);
        }
        catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        return null;
    }
}